@@ -12,12 +12,12 @@
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
- *_
+ *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- * $Id: bacon-video-widget-gst.c,v 1.87 2004/10/06 12:06:25 rbultje Exp $
+ * $Id: bacon-video-widget-gst.c,v 1.96 2004/10/11 15:53:13 rbultje Exp $
  *
  */
 
@@ -46,6 +46,7 @@
 #include "scrsaver.h"
 #include "video-utils.h"
 #include "gststreaminfo.h"
+#include "gstscreenshot.h"
 
 #define DEFAULT_HEIGHT 420
 #define DEFAULT_WIDTH 315
@@ -187,12 +188,29 @@ static GtkWidgetClass *parent_class = NULL;
 
 static int bvw_table_signals[LAST_SIGNAL] = { 0 };
 
+static void
+get_media_size (BaconVideoWidget *bvw, gint *width, gint *height)
+{
+  if (GST_STATE (bvw->priv->play) >= GST_STATE_PAUSED) {
+    *width = bvw->priv->video_width;
+    *height = bvw->priv->video_height;
+  } else {
+    if (bvw->priv->logo_pixbuf != NULL) {
+      *width = gdk_pixbuf_get_width (bvw->priv->logo_pixbuf);
+      *height = gdk_pixbuf_get_height (bvw->priv->logo_pixbuf);
+    } else {
+      *width = bvw->priv->init_width;
+      *height = bvw->priv->init_height;
+    }
+  }
+}
+
 static void
 bacon_video_widget_realize (GtkWidget * widget)
 {
   BaconVideoWidget *bvw = BACON_VIDEO_WIDGET (widget);
   GdkWindowAttr attributes;
-  gint attributes_mask;
+  gint attributes_mask, w, h;
 
   /* Creating our widget's window */
   attributes.window_type = GDK_WINDOW_CHILD;
@@ -232,6 +250,10 @@ bacon_video_widget_realize (GtkWidget * widget)
   gtk_style_set_background (widget->style, widget->window, GTK_STATE_NORMAL);
 
   GTK_WIDGET_SET_FLAGS (widget, GTK_REALIZED);
+
+  /* nice hack to show the logo fullsize, while still being resizable */
+  get_media_size (BACON_VIDEO_WIDGET (widget), &w, &h);
+  totem_widget_set_preferred_size (widget, w, h);
 }
 
 static gboolean
@@ -247,19 +269,10 @@ bacon_video_widget_expose_event (GtkWidget *widget, GdkEventExpose *event)
 			GST_IS_X_OVERLAY (bvw->priv->xoverlay),
 			FALSE);
 
-  if (GST_STATE (bvw->priv->play) >= GST_STATE_PAUSED) {
-    window = GDK_WINDOW_XWINDOW (bvw->priv->video_window);
-  } else {
-    window = 0;
-  }
+  window = GDK_WINDOW_XWINDOW (bvw->priv->video_window);
 
   gdk_draw_rectangle (widget->window, widget->style->black_gc, TRUE, 0, 0,
 		      widget->allocation.width, widget->allocation.height);
-  gdk_draw_rectangle (bvw->priv->video_window, widget->style->black_gc, TRUE,
-		      bvw->priv->video_window_allocation.x,
-		      bvw->priv->video_window_allocation.y,
-		      bvw->priv->video_window_allocation.width,
-		      bvw->priv->video_window_allocation.height);
   gst_x_overlay_set_xwindow_id (bvw->priv->xoverlay, window);
 
   if (GST_STATE (bvw->priv->play) >= GST_STATE_PAUSED) {
@@ -295,6 +308,12 @@ bacon_video_widget_expose_event (GtkWidget *widget, GdkEventExpose *event)
 
     gdk_pixbuf_unref (logo);
   }
+  else {
+    gdk_draw_rectangle (bvw->priv->video_window, widget->style->black_gc, TRUE,
+		      0, 0,
+		      bvw->priv->video_window_allocation.width,
+		      bvw->priv->video_window_allocation.height);
+  }
   
   return TRUE;
 }
@@ -305,20 +324,8 @@ bacon_video_widget_size_request (GtkWidget * widget,
 {
   BaconVideoWidget *bvw = BACON_VIDEO_WIDGET (widget);
 
-  /* FIXME: intial width, this is minimal, but not "recommended"
-   * (media size) */
-  if (GST_STATE (bvw->priv->play) >= GST_STATE_PAUSED) {
-    requisition->width = bvw->priv->video_width;
-    requisition->height = bvw->priv->video_height;
-  } else {
-    if (bvw->priv->logo_pixbuf != NULL) {
-      requisition->width = gdk_pixbuf_get_width (bvw->priv->logo_pixbuf);
-      requisition->height = gdk_pixbuf_get_height (bvw->priv->logo_pixbuf);
-    } else {
-      requisition->width = bvw->priv->init_width;
-      requisition->height = bvw->priv->init_height;
-    }
-  }
+  requisition->width = bvw->priv->init_width;
+  requisition->height = bvw->priv->init_height;
 }
 
 static void
@@ -525,7 +532,6 @@ shrink_toplevel (BaconVideoWidget * bvw)
   GtkWidget *toplevel;
   toplevel = gtk_widget_get_toplevel (GTK_WIDGET (bvw));
   gtk_window_resize (GTK_WINDOW (toplevel), 1, 1);
-  gtk_widget_queue_resize (GTK_WIDGET (bvw));
 }
 
 static gboolean
@@ -545,38 +551,18 @@ bacon_video_widget_signal_idler (BaconVideoWidget *bvw)
     {
       case ASYNC_VIDEO_SIZE:
         {
-          GList *streaminfo = NULL;
-
-          g_object_get (G_OBJECT (bvw->priv->play), "stream-info",
-			&streaminfo, NULL);
-          for ( ; streaminfo != NULL; streaminfo = streaminfo->next) {
-            GstStreamInfo *info = streaminfo->data;
-
-            switch (info->type) {
-              case GST_STREAM_TYPE_VIDEO: {
-                GstStructure *s;
-
-                s = gst_caps_get_structure (GST_PAD_CAPS (info->pad), 0);
-                if (s) {
-                  gst_structure_get_double (s, "framerate", &bvw->priv->video_fps);
-                  gst_structure_get_int (s, "width", &bvw->priv->video_width);
-                  gst_structure_get_int (s, "height", &bvw->priv->video_height);
-                  /* FIXME: a-s-r */
-                }
-                break;
-              }
-              default:
-                break;
-            }
-          }
-
           g_signal_emit (G_OBJECT (bvw), bvw_table_signals[SIGNAL_GOT_METADATA], 0, NULL);
 
           if (bvw->priv->auto_resize) {
+            gint w, h;
+
             shrink_toplevel (bvw);
+            get_media_size (bvw, &w, &h);
+            totem_widget_set_preferred_size (GTK_WIDGET (bvw), w, h);
           } else {
             bacon_video_widget_size_allocate (GTK_WIDGET (bvw),
 					      &GTK_WIDGET (bvw)->allocation);
+            bacon_video_widget_expose_event (GTK_WIDGET (bvw), NULL);
           }
 
           break;
@@ -617,23 +603,20 @@ bacon_video_widget_signal_idler (BaconVideoWidget *bvw)
         }
       case ASYNC_FOUND_TAG:
         {
-          GstTagList *tag_list = signal->signal_data.found_tag.tag_list;
+          GstTagList *tag_list = signal->signal_data.found_tag.tag_list,
+		*result;
 
-          if (tag_list) {
-            if (bvw->priv->tagcache) {
-              GstTagList *taglist;
+          result = gst_tag_list_merge (bvw->priv->tagcache, tag_list,
+				       GST_TAG_MERGE_APPEND);
+          if (tag_list)
+            gst_tag_list_free (tag_list);
+          if (bvw->priv->tagcache)
+            gst_tag_list_free (bvw->priv->tagcache);
 
-              taglist = gst_tag_list_merge (bvw->priv->tagcache, tag_list,
-					    GST_TAG_MERGE_APPEND);
-              gst_tag_list_free (tag_list);
-              gst_tag_list_free (bvw->priv->tagcache);
-              bvw->priv->tagcache = taglist;
-            } else {
-              bvw->priv->tagcache = tag_list;
-            }
-            g_signal_emit (G_OBJECT (bvw), bvw_table_signals[SIGNAL_GOT_METADATA],
-                           0, NULL);
-          }
+          bvw->priv->tagcache = result;
+          g_signal_emit (G_OBJECT (bvw),
+			 bvw_table_signals[SIGNAL_GOT_METADATA],
+			 0, NULL);
           break;
         }
       case ASYNC_EOS:
@@ -680,7 +663,7 @@ got_video_size (GstElement * play, guint width, guint height,
   
   g_return_if_fail (bvw != NULL);
   g_return_if_fail (BACON_IS_VIDEO_WIDGET (bvw));
-  
+
   signal = g_new0 (BVWSignal, 1);
   signal->signal_id = ASYNC_VIDEO_SIZE;
   signal->signal_data.video_size.width = width;
@@ -785,6 +768,39 @@ cb_iterate (BaconVideoWidget *bvw)
   return res;
 }
 
+static void
+caps_set (GObject * obj,
+    GParamSpec * pspec, BaconVideoWidget * bvw)
+{
+  GstPad *pad = GST_PAD (obj);
+  GstStructure *s;
+
+  if (!GST_PAD_CAPS (pad))
+    return;
+
+  s = gst_caps_get_structure (GST_PAD_CAPS (pad), 0);
+  if (s) {
+    const GValue *par;
+
+    gst_structure_get_double (s, "framerate", &bvw->priv->video_fps);
+    gst_structure_get_int (s, "width", &bvw->priv->video_width);
+    gst_structure_get_int (s, "height", &bvw->priv->video_height);
+    if ((par = gst_structure_get_value (s,
+                   "pixel-aspect-ratio"))) {
+      gint num = gst_value_get_fraction_numerator (par),
+          den = gst_value_get_fraction_denominator (par);
+
+      if (num > den)
+        bvw->priv->video_width *= (gfloat) num / den;
+      else
+        bvw->priv->video_height *= (gfloat) den / num;
+    }
+  }
+
+  /* and disable ourselves */
+  g_signal_handlers_disconnect_by_func (pad, caps_set, bvw);
+}
+
 static void
 state_change (GstElement *play, GstElementState old_state,
 	      GstElementState new_state, BaconVideoWidget *bvw)
@@ -813,9 +829,18 @@ state_change (GstElement *play, GstElementState old_state,
         case GST_STREAM_TYPE_AUDIO:
           bvw->priv->media_has_audio = TRUE;
           break;
-        case GST_STREAM_TYPE_VIDEO:
+        case GST_STREAM_TYPE_VIDEO: {
+
+          /* handle explicit caps as well - they're set later */
+          if (GST_PAD_CAPS (info->pad))
+            caps_set (G_OBJECT (info->pad), NULL, bvw);
+          else
+            g_signal_connect (info->pad, "notify::caps",
+                G_CALLBACK (caps_set), bvw);
+
           bvw->priv->media_has_video = TRUE;
           break;
+        }
         default:
           break;
       }
@@ -831,6 +856,9 @@ state_change (GstElement *play, GstElementState old_state,
         gst_tag_list_free (bvw->priv->tagcache);
         bvw->priv->tagcache = NULL;
       }
+
+    bvw->priv->video_width = 0;
+    bvw->priv->video_height = 0;
   }
 }
 
@@ -1035,6 +1063,10 @@ bacon_video_widget_open (BaconVideoWidget * bvw, const gchar * mrl,
   g_return_val_if_fail (bvw->priv->play != NULL, FALSE);
   g_return_val_if_fail (bvw->priv->mrl == NULL, FALSE);
 
+  /* hmm... */
+  if (bvw->priv->mrl && !strcmp (bvw->priv->mrl, mrl))
+    return TRUE;
+
   g_free (bvw->priv->mrl);
   bvw->priv->mrl = g_strdup (mrl);
 
@@ -1188,7 +1220,11 @@ bacon_video_widget_set_logo (BaconVideoWidget * bvw, gchar * filename)
                filename, error->message);
     g_error_free (error);
   } else {
+    gint w, h;
+
     shrink_toplevel (bvw);
+    get_media_size (bvw, &w, &h);
+    totem_widget_set_preferred_size (GTK_WIDGET (bvw), w, h);
   }
 }
 
@@ -1420,7 +1456,7 @@ bacon_video_widget_get_visuals_list (BaconVideoWidget * bvw)
 gboolean
 bacon_video_widget_set_visuals (BaconVideoWidget * bvw, const char *name)
 {
-  //gboolean paused = FALSE;
+  /*gboolean paused = FALSE;*/
 
   g_return_val_if_fail (bvw != NULL, FALSE);
   g_return_val_if_fail (BACON_IS_VIDEO_WIDGET (bvw), FALSE);
@@ -1539,13 +1575,30 @@ bacon_video_widget_get_aspect_ratio (BaconVideoWidget *bvw)
 void
 bacon_video_widget_set_scale_ratio (BaconVideoWidget * bvw, gfloat ratio)
 {
+  gint w, h;
+
   g_return_if_fail (bvw != NULL);
   g_return_if_fail (BACON_IS_VIDEO_WIDGET (bvw));
   g_return_if_fail (GST_IS_ELEMENT (bvw->priv->play));
 
-//  gst_video_widget_set_scale (bvw->priv->vw, ratio);
-//  gst_video_widget_set_scale_override (bvw->priv->vw, TRUE);
+  get_media_size (bvw, &w, &h);
+  if (ratio == 0) {
+    if (totem_ratio_fits_screen (bvw->priv->video_window, w, h, 2.0))
+      ratio = 2.0;
+    else if (totem_ratio_fits_screen (bvw->priv->video_window, w, h, 1.0))
+      ratio = 1.0;
+    else if (totem_ratio_fits_screen (bvw->priv->video_window, w, h, 0.5))
+      ratio = 0.5;
+    else
+      return;
+  } else {
+    if (!totem_ratio_fits_screen (bvw->priv->video_window, w, h, ratio))
+      return;
+  }
+  w = (gfloat) w * ratio;
+  h = (gfloat) h * ratio;
   shrink_toplevel (bvw);
+  totem_widget_set_preferred_size (GTK_WIDGET (bvw), w, h);
 }
 
 int
@@ -1718,21 +1771,57 @@ bacon_video_widget_is_seekable (BaconVideoWidget * bvw)
 gboolean
 bacon_video_widget_can_play (BaconVideoWidget * bvw, MediaType type)
 {
+  gboolean res;
+
   g_return_val_if_fail (bvw != NULL, FALSE);
   g_return_val_if_fail (BACON_IS_VIDEO_WIDGET (bvw), FALSE);
   g_return_val_if_fail (GST_IS_ELEMENT (bvw->priv->play), FALSE);
 
-  return FALSE;
+  switch (type) {
+    case MEDIA_TYPE_CDDA:
+      res = TRUE;
+      break;
+    case MEDIA_TYPE_DVD:
+    case MEDIA_TYPE_VCD:
+    default:
+      res = FALSE;
+      break;
+  }
+
+  return res;
 }
 
 G_CONST_RETURN gchar **
 bacon_video_widget_get_mrls (BaconVideoWidget * bvw, MediaType type)
 {
+  const gchar **mrls;
+
   g_return_val_if_fail (bvw != NULL, NULL);
   g_return_val_if_fail (BACON_IS_VIDEO_WIDGET (bvw), NULL);
   g_return_val_if_fail (GST_IS_ELEMENT (bvw->priv->play), NULL);
 
-  return NULL;
+  switch (type) {
+    case MEDIA_TYPE_CDDA: {
+      static const gchar *uri[] = { "cdda://", NULL };
+      mrls = uri;
+      break;
+    }
+    case MEDIA_TYPE_VCD: {
+      static const gchar *uri[] = { "vcd://", NULL };
+      mrls = uri;
+      break;
+    }
+    case MEDIA_TYPE_DVD: {
+      static const gchar *uri[] = { "dvd://", NULL };
+      mrls = uri;
+      break;
+    }
+    default:
+      mrls = NULL;
+      break;
+  }
+
+  return mrls;
 }
 
 static void
@@ -1745,7 +1834,7 @@ bacon_video_widget_get_metadata_string (BaconVideoWidget * bvw,
 
   g_value_init (value, G_TYPE_STRING);
 
-  if (bvw->priv->play == NULL)
+  if (bvw->priv->play == NULL || bvw->priv->tagcache == NULL)
     {
       g_value_set_string (value, NULL);
       return;
@@ -1774,7 +1863,8 @@ bacon_video_widget_get_metadata_string (BaconVideoWidget * bvw,
 				     GST_TAG_AUDIO_CODEC, &string);
       break;
     case BVW_INFO_CDINDEX:
-      res = FALSE;
+      res = gst_tag_list_get_string (bvw->priv->tagcache,
+				     "discid", &string);
       break;
     default:
       g_assert_not_reached ();
@@ -1907,16 +1997,88 @@ bacon_video_widget_can_get_frames (BaconVideoWidget * bvw, GError ** error)
   g_return_val_if_fail (bvw != NULL, FALSE);
   g_return_val_if_fail (BACON_IS_VIDEO_WIDGET (bvw), FALSE);
   g_return_val_if_fail (GST_IS_ELEMENT (bvw->priv->play), FALSE);
-  return FALSE;
+
+  /* check for version */
+  if (!g_object_class_find_property (
+           G_OBJECT_GET_CLASS (bvw->priv->play), "frame"))
+    return FALSE;
+
+  /* check for video */
+  return bvw->priv->media_has_video;
+}
+
+static void
+destroy_pixbuf (guchar *pix, gpointer data)
+{
+  gst_buffer_unref (GST_BUFFER (data));
 }
 
 GdkPixbuf *
 bacon_video_widget_get_current_frame (BaconVideoWidget * bvw)
 {
+  GstBuffer *buf = NULL;
+  const GList *streaminfo = NULL;
+  GstCaps *from = NULL;
+  GdkPixbuf *pixbuf;
+
   g_return_val_if_fail (bvw != NULL, NULL);
   g_return_val_if_fail (BACON_IS_VIDEO_WIDGET (bvw), NULL);
   g_return_val_if_fail (GST_IS_ELEMENT (bvw->priv->play), NULL);
-  return NULL;
+
+  /* no video info */
+  if (!bvw->priv->video_width || !bvw->priv->video_height)
+    return NULL;
+
+  /* get frame */
+  g_object_get (G_OBJECT (bvw->priv->play), "frame", &buf, NULL);
+  if (!buf)
+    return NULL;
+
+  /* take our own reference, because we will eat it */
+  gst_buffer_ref (buf);
+
+  /* get video size etc. */
+  g_object_get (G_OBJECT (bvw->priv->play),
+      "stream-info", &streaminfo, NULL);
+  for (; streaminfo != NULL; streaminfo = streaminfo->next) {
+    GstStreamInfo *info = streaminfo->data;
+
+    if (info->type == GST_STREAM_TYPE_VIDEO) {
+      from = gst_caps_copy (GST_PAD_CAPS (info->pad));
+      break;
+    }
+  }
+  if (!from)
+    return NULL;
+
+  /* convert to our own wanted format */
+  buf = bvw_frame_conv_convert (buf, from,
+      gst_caps_new_simple ("video/x-raw-rgb",
+          "bpp", G_TYPE_INT, 24,
+          "depth", G_TYPE_INT, 24,
+          "width", G_TYPE_INT, bvw->priv->video_width,
+          "height", G_TYPE_INT, bvw->priv->video_height,
+          "framerate", G_TYPE_DOUBLE, bvw->priv->video_fps,
+          "pixel-aspect-ratio", GST_TYPE_FRACTION, 1, 1,
+          "endianness", G_TYPE_INT, G_BIG_ENDIAN,
+          "red_mask", G_TYPE_INT, 0xff0000,
+          "green_mask", G_TYPE_INT, 0x00ff00,
+          "blue_mask", G_TYPE_INT, 0x0000ff, NULL));
+  if (!buf)
+    return NULL;
+
+  /* create pixbuf from that - use our own destroy function */
+  pixbuf = gdk_pixbuf_new_from_data (GST_BUFFER_DATA (buf),
+			GDK_COLORSPACE_RGB, FALSE,
+			8, bvw->priv->video_width,
+			bvw->priv->video_height,
+			3 * bvw->priv->video_width,
+			destroy_pixbuf, buf);
+  if (!pixbuf)
+    gst_buffer_unref (buf);
+
+  /* that's all */
+  return pixbuf;
 }
 
 /* =========================================== */
@@ -1964,29 +2126,30 @@ bacon_video_widget_new (int width, int height,
   }
   
   bvw->priv->media_device = g_strdup ("/dev/dvd");
-  if (width <= 0 || height <= 0) {
-    width = DEFAULT_WIDTH;
-    height = DEFAULT_HEIGHT;
-  }
-  bvw->priv->init_width = width;
-  bvw->priv->init_height = height;
+  bvw->priv->init_width = 240;
+  bvw->priv->init_height = 180;
 
   bvw->priv->cursor_shown = TRUE;
   bvw->priv->logo_mode = TRUE;
   bvw->priv->auto_resize = TRUE;
 
-  audio_sink = gst_gconf_get_default_audio_sink ();
-  if (!GST_IS_ELEMENT (audio_sink))
-    {
-      g_message ("failed to render default audio sink from gconf");
-      return NULL;
-    }
-  video_sink = gst_gconf_get_default_video_sink ();
-  if (!GST_IS_ELEMENT (video_sink))
-    {
-      g_message ("failed to render default video sink from gconf");
-      return NULL;
-    }
+  if (!null_out) {
+    audio_sink = gst_gconf_get_default_audio_sink ();
+    if (!GST_IS_ELEMENT (audio_sink))
+      {
+        g_message ("failed to render default audio sink from gconf");
+        return NULL;
+      }
+    video_sink = gst_gconf_get_default_video_sink ();
+    if (!GST_IS_ELEMENT (video_sink))
+      {
+        g_message ("failed to render default video sink from gconf");
+        return NULL;
+      }
+  } else {
+    video_sink = gst_element_factory_make ("fakesink", "fakevideosink");
+    audio_sink = gst_element_factory_make ("fakesink", "fakeaudiosink");
+  }
 
   g_object_set (G_OBJECT (bvw->priv->play), "video-sink",
 		video_sink, NULL);
@@ -2005,7 +2168,7 @@ bacon_video_widget_new (int width, int height,
 		    (GtkSignalFunc) got_error, (gpointer) bvw);
 
   /* We try to get an element supporting XOverlay interface */
-  if (GST_IS_BIN (bvw->priv->play)) {
+  if (!null_out && GST_IS_BIN (bvw->priv->play)) {
     GstElement *element = NULL;
     GList *elements = NULL, *l_elements = NULL;
     
